// user13 Evolving Generative Art Fragment Shader                                                      
//----------------------------------------// shader name length of 40 characters

precision mediump float;

varying vec2 tcoord;

uniform float time;                     // System time for animation (in seconds)
uniform vec2 tres;                      // Center of the screen

uniform vec4 seed;                      // random value between 0 and 1
uniform vec4 audio;                     // Low, Mid, Hi energy average
uniform vec4 color;
uniform vec4 par_a;                     // Normalised ( 0.0 to 1.0 )
uniform vec4 par_b;                     // Normalised ( 0.0 to 1.0 )

uniform sampler2D tex[8];               // Array of tex samplers
uniform int tex_l;                      // number of loaded texures

float f0 = mix(0.1, 0.95, par_a[0]);    // f0: :
float f1 = mix(0.05, 0.95, par_a[1]);   // f1: :
float f2 = mix(0.05, 0.95, par_a[2]);   // f2: :

// Noise function
float noise(vec2 st)
{
    return fract(sin(dot(st.xy, vec2(12.9898,78.233))) * 43758.5453123);
}

void main()
{
    vec2 uv = (gl_FragCoord.xy - tres) / min(tres.x, tres.y);
    uv *= 1.0; //zppm
    // UV transformation (rotation)
    // Generate evolving patterns
    float n = noise(uv * 5.0 + time);
    float pattern = sin(uv.x * 10.0 + n) * cos(uv.y * 10.0 + n);
    // Create circular waves
    float dist = length(uv);
    float wave = sin(dist * 20.0 - time * 5.0) * 0.5 + 0.5;
    // Combine patterns
    float combined = mix(pattern, wave, 0.5);
    // Generate color_out
    vec3 color_out = vec3(
        0.5 + 0.5 * sin(combined * 5.0 + time),
        0.5 + 0.5 * sin(combined * 7.0 + time * 1.1),
        0.5 + 0.5 * sin(combined * 9.0 + time * 1.2)
    );
    // Apply effects based on uniforms
    color_out = mix(color_out, 1.0 - color_out, par_a[0]);  // Color inversion
    color_out *= 1.0 - (par_a[0] * 0.5);  // Darkness control
    color_out = mix(color_out, vec3(dot(color_out, vec3(0.299, 0.587, 0.114))), par_a[1]);  // Saturation control

    gl_FragColor = vec4(color_out, color.a);
}